#include <iostream>
#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <vector>
#include <algorithm>
#include <sstream>

using namespace std;

/* assumption data structure */
struct assumption
{
    string name;  /* unique name of assumption */
    
    /* constructor */
    assumption() {}
    assumption(string name) : name(name) {}
    
    bool operator == (const assumption& as) const
    {
        return name.compare(as.name) == 0;
    }
    
    bool operator < (const assumption& as) const
    {
        return name.compare(as.name) < 0;
    }
    
    void print()
    {
        cout << "(" << name << ")";
    }
};

/* environment data structure */
struct environment
{
    vector<assumption> as;  /* variable */
    
    /* constructor */
    environment() {}
    environment(vector<assumption> as) : as(as) {}
    
    /* add assumption to environment */
    void add(assumption x)
    {
        as.push_back(x);
        return;
    }
    
    void sort_assumption()
    {
        sort(as.begin(), as.end());
        return;
    }
    
    bool is_contain(assumption key)
    {
        return binary_search(as.begin(), as.end(), key);
    }
};

/* node data structure */
struct node
{
    vector<environment> label;  /* label of node */
    
    /* constructor */
    node() {}
    node(vector<environment> label) : label(label) {}
    
    /* add environment to label of node */
    void update_label(vector<environment> lb)
    {
        label.clear();
        for (int i = 0; i < lb.size(); ++i)
        {
            label.push_back(lb[i]);
        }
        return;
    }
    
    /*  */
    void add_environment(environment env)
    {
        label.push_back(env);
        return;
    }
    
    /* return label of node */
    vector<environment> get_label()
    {
        return label;
    }  
};

/* atms data structure */
struct atms
{
    vector<environment> nogood;     /* list of nogood environment */
    
    atms() {}   /* constructor */
    
    /* add nogood environment */
    void add_nogood(environment env)
    {
        nogood.push_back(env);
        return;
    }
    
    /* justify an environment, return true if nogood */
    bool justify(environment env)
    {
        env.sort_assumption();
        for (int i = 0; i < nogood.size(); ++i)
        {
            bool check = true;
            for (int j = 0; j < nogood[i].as.size(); ++j)
            {
                if (!env.is_contain(nogood[i].as[j]))
                {
                    check = false;
                    break;
                }
            }
            if (check) return true;
        }
        return false;
    }
};

/* convert an integer to string */
string int_to_string(int n)
{
    stringstream ss;
    ss << n;
    return ss.str();
}

int main (int argc, char const *argv[])
{
    /* test for each data set */
    char color[] = {'R', 'G', 'B', 'Y'};
    int num_area;       /* number of areas */
    int num_neighbor;   /* number of pairs of neighbored areas */
    atms atm = atms();  /* create ATMS */
    
    /* read map data from file */
    cin >> num_area >> num_neighbor;
    
    vector<assumption> as(4 * (num_area + 1));
    
    /* create all assumptions */
    for (int i = 1; i <= num_area; ++i)
    {
        for (int j = 0; j < 4; ++j)
        {
            string name = int_to_string(i) + color[j];
            as[4 * i + j] = assumption(name);
        }
    }
    
    /* notify all nogood environments to ATMS */
    vector<assumption> nogood_tmp;
    int area_x, area_y;
    for(int t = 0; t < num_neighbor; ++t)
    {
        cin >> area_x >> area_y;
        for (int i = 0; i < 4; ++i)
        {
            nogood_tmp.clear();
            nogood_tmp.push_back(as[4 * area_x + i]);
            nogood_tmp.push_back(as[4 * area_y + i]);
            atm.add_nogood(environment(nogood_tmp));
        }
    }
    
    /* update label */
    node goal = node();
    for (int i = 0; i < 4; ++i)
    {
        environment env = environment();
        env.as.push_back(as[4 + i]);
        goal.add_environment(env);
    }
    for (int i = 2; i <= num_area; ++i)
    {
        vector<environment> label = goal.get_label();
        vector<environment> tmp;
        
        for (int k = 0; k < 4; ++k)
        {
            for(int j = 0; j < label.size(); ++j)
            {
                environment env = label[j];
                env.as.push_back(as[4 * i + k]);
                if (atm.justify(env)) continue;
                tmp.push_back(env);
            }
        }
        goal.update_label(tmp);
    }
    
    /* print all solutions */
    vector<environment> solution = goal.get_label();
    cout << "All solutions of 4 colors problem : " << solution.size() << " solutions" << endl;
    for (int i = 0; i < solution.size(); ++i)
    {
        for (int j = 0; j < solution[i].as.size(); ++j)
        {
            solution[i].as[j].print();
        }
        cout << endl;
    }
    return 0;
}